@@ -23,7 +23,7 @@ import re, os
 
 
 # The current version of rdiff-backup
-version = "0.12.6"
+version = "0.12.7"
 
 # If this is set, use this value in seconds as the current time
 # instead of reading it from the clock.
@@ -135,7 +135,7 @@ compression = 1
 # to .snapshots and .diffs).  The second below will be the
 # compiled version of the first.
 no_compression_regexp_string = "(?i).*\\.(gz|z|bz|bz2|tgz|zip|rpm|deb|" \
-						"jpg|gif|png|jp2|mp3|ogg|avi|wmv|mpeg|mpg|rm|mov)$"
+						"jpg|gif|png|jp2|mp3|ogg|avi|wmv|mpeg|mpg|rm|mov|flac|shn)$"
 no_compression_regexp = None
 
 # If true, filelists and directory statistics will be split on
