@@ -20,7 +20,7 @@
 """Start (and end) here - read arguments, set global settings, etc."""
 
 from __future__ import generators
-import getopt, sys, re, os
+import getopt, sys, re, os, cStringIO
 from log import Log, LoggerError, ErrorLog
 import Globals, Time, SetConnections, selection, robust, rpath, \
 	   manage, backup, connection, restore, FilenameMapping, \
@@ -42,6 +42,10 @@ def parse_cmdlineoptions(arglist):
 		try: return open(filename, "r")
 		except IOError: Log.FatalError("Error opening file %s" % filename)
 
+	def normalize_path(path):
+		"""Used below to normalize the security paths before setting"""
+		return rpath.RPath(Globals.local_connection, path).normalize().path
+
 	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
 		 ["backup-mode", "calculate-average", "chars-to-quote=",
 		  "check-destination-dir", "current-time=", "exclude=",
@@ -128,13 +132,13 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "--remove-older-than":
 			remove_older_than_string = arg
 			action = "remove-older-than"
-		elif opt == "--restrict": Globals.restrict_path = arg
+		elif opt == "--restrict": Globals.restrict_path = normalize_path(arg)
 		elif opt == "--restrict-read-only":
 			Globals.security_level = "read-only"
-			Globals.restrict_path = arg
+			Globals.restrict_path = normalize_path(arg)
 		elif opt == "--restrict-update-only":
 			Globals.security_level = "update-only"
-			Globals.restrict_path = arg
+			Globals.restrict_path = normalize_path(arg)
 		elif opt == "-s" or opt == "--server":
 			action = "server"
 			Globals.server = 1
@@ -320,14 +324,16 @@ option.""" % rpout.path)
 
 def backup_warn_if_infinite_regress(rpin, rpout):
 	"""Warn user if destination area contained in source area"""
-	if rpout.conn is rpin.conn: # it's meaningful to compare paths
-		if ((len(rpout.path) > len(rpin.path)+1 and
-			 rpout.path[:len(rpin.path)] == rpin.path and
-			 rpout.path[len(rpin.path)] == '/') or
-			(rpin.path == "." and rpout.path[0] != '/' and
-			 rpout.path[:2] != '..')):
-			# Just a few heuristics, we don't have to get every case
-			if Globals.backup_reader.Globals.select_source.Select(rpout): Log(
+	# Just a few heuristics, we don't have to get every case
+	if rpout.conn is not rpin.conn: return
+	if len(rpout.path) <= len(rpin.path)+1: return
+	if rpout.path[:len(rpin.path)+1] != rpin.path + '/': return
+
+	relative_rpout_comps = tuple(rpout.path[len(rpin.path)+1:].split('/'))
+	relative_rpout = rpin.new_index(relative_rpout_comps)
+	if not Globals.select_mirror.Select(relative_rpout): return
+
+	Log(
 """Warning: The destination directory '%s' may be contained in the
 source directory '%s'.  This could cause an infinite regress.  You
 may need to use the --exclude option.""" % (rpout.path, rpin.path), 2)
@@ -408,16 +414,26 @@ def restore_common(rpin, target, time):
 	Log("Restore ended", 4)
 
 def restore_set_select(mirror_rp, target):
-	"""Set the selection iterator on mirror side from command line args
+	"""Set the selection iterator on both side from command line args
 
-	Here we set the selector on the mirror side, because that is where
-	we will be filtering, but the pathnames are relative to the target
-	directory.
+	We must set both sides because restore filtering is different from
+	select filtering.  For instance, if a file is excluded it should
+	not be deleted from the target directory.
+
+	The StringIO stuff is because filelists need to be read and then
+	duplicated, because we need two copies of them now.
 
 	"""
+	def fp2string(fp):
+		buf = fp.read()
+		assert not fp.close()
+		return buf
+	select_data = map(fp2string, select_files)
 	if select_opts: 
 		mirror_rp.conn.restore.MirrorStruct.set_mirror_select(
-			target, select_opts, *select_files)
+			target, select_opts, *map(cStringIO.StringIO, select_data))
+		target.conn.restore.TargetStruct.set_target_select(
+			target, select_opts, *map(cStringIO.StringIO, select_data))
 
 def restore_start_log(rpin, target, time):
 	"""Open restore log file, log initial message"""
