@@ -724,7 +724,9 @@ class RPath(RORPath):
 		log.Log("Deleting %s" % self.path, 7)
 		if self.isdir():
 			try: self.rmdir()
-			except os.error: shutil.rmtree(self.path)
+			except os.error:
+				if Globals.fsync_directories: self.fsync()
+				self.conn.shutil.rmtree(self.path)
 		else: self.conn.os.unlink(self.path)
 		self.setdata()
 
